/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <memory>

#include "RobotAPI/libraries/aron/core/navigator/data/NavigatorFactory.h"
#include "RobotAPI/libraries/aron/core/navigator/data/primitive/String.h"
#include <RobotAPI/interface/aron/Aron.h>
#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>

#include "Navigation/libraries/core/types.h"
#include "Navigation/libraries/factories/NavigationStackFactory.h"
#include "Navigation/libraries/global_planning/GlobalPlanner.h"
#include "Navigation/libraries/local_planning/LocalPlanner.h"
#include "Navigation/libraries/safety_control/SafetyController.h"
#include "Navigation/libraries/trajectory_control/TrajectoryController.h"
#include <type_traits>

namespace armarx::nav::client
{
    struct GeneralConfig
    {
        struct VelocityLimit
        {
            float linear;
            float angular;
        };

        // if the desired velocity is below this threshold, the platform will not be moved
        VelocityLimit minVel{0.F, 0.F};

        //! max ...
        VelocityLimit maxVel;

        core::NavigationFrames navigationFrame = core::NavigationFrames::Absolute;

        virtual aron::datanavigator::DictNavigatorPtr toAron() const;

    };

    class NavigationStackConfig
    {
    public:
        NavigationStackConfig() = default;

        NavigationStackConfig& configureGeneral(const GeneralConfig& cfg);

        NavigationStackConfig& configureGlobalPlanner(const glob_plan::GlobalPlannerParams& params);

        NavigationStackConfig& configureLocalPlanner(const loc_plan::LocalPlannerParams& params);

        NavigationStackConfig&
        configureTrajectoryController(const traj_ctrl::TrajectoryControllerParams& params);

        NavigationStackConfig&
        configureSafetyController(const safe_ctrl::SafetyControllerParams& params);

        aron::data::AronDictPtr toAron() const;

    protected:
    private:
        aron::datanavigator::DictNavigator dict;
    };
} // namespace armarx::nav::client
