/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/VirtualRobot.h>

#include "RobotAPI/libraries/aron/core/navigator/data/container/Dict.h"

#include "Navigation/libraries/core/DynamicScene.h"
#include "Navigation/libraries/core/StaticScene.h"
#include "Navigation/libraries/core/Trajectory.h"
#include "Navigation/libraries/core/types.h"
#include "core.h"

namespace armarx::nav::glob_plan
{

    struct GlobalPlannerParams
    {
        virtual Algorithms algorithm() const = 0;
        virtual aron::datanavigator::DictNavigatorPtr toAron() const = 0;
    };

    class GlobalPlanner
    {
    public:
        GlobalPlanner(const core::Scene& context);
        virtual ~GlobalPlanner() = default;

        virtual core::TrajectoryPtr plan(const core::Pose& goal) = 0;

    protected:
        const core::Scene context;

    private:
    };

    using GlobalPlannerPtr = std::shared_ptr<GlobalPlanner>;
} // namespace armarx::nav::glob_plan
